#include <afx.h>
#include <afxdb.h>
#include <winnt.h>
#include <winbase.h>


class myRS : public CRecordset
{
public:
    DECLARE_DYNAMIC(myRS);

    myRS(CDatabase* pdb = NULL)
        : CRecordset(pdb)
    {
        m_ID = 0;
        m_c2_s = _T("");

        m_nFields = 2;                     // should be last field. Must be updated if new fields added
    }
    virtual void DoFieldExchange(CFieldExchange* pFX)
    {
        pFX->SetFieldType(CFieldExchange::outputColumn);
        RFX_Long(pFX, _T("[ID]"), m_ID);
        RFX_Text(pFX, _T("[c2_s]"), m_c2_s);
    }
    virtual CString GetDefaultConnect()
    {
        return "ODBC;DSN=test1";
    }
    virtual CString GetDefaultSQL()
    {
        return _T("SELECT `ID`,`c2_s` FROM tab1");
    }

    long    m_ID;
    CString m_c2_s;
};
IMPLEMENT_DYNAMIC(myRS, CRecordset);

//----------------------------------------------------------------
int main(int argc, char ** argv)
{
// odbc mysql driver 3.51.20
// Issue:
//    very large data set (million records)
//    moving forward thru all records produces big cache file on %TEMP% dir
//    many active apps/threads moving forward thru big result will fillup free disk space
//
// Goal: disable cache creation (All tests with  "Don't Cache Result (forward only cursors)" set in odbc driver options)
//          and being able to Add/Update records during forward navigation.
// 
// case1: db.open, tab.open(forward), Edit(), Update() -
//    Error: Access Violation in MYODBC3.dll (from the stack)
//
// case2: db.open, tab.open(forward), AddNew(), Update() -
//    Error: exception from Update() - Commands out of sync; you can't run this command now
//

    LPCSTR             locus = "";
    try{
//        int caseid = 1; LPCSTR casedesc = "Modify record";
        int caseid = 2; LPCSTR casedesc = "Add record";

        printf("case %d %s\n", caseid, casedesc);

        CDatabase  * db = NULL;
        if(1){
            db = new CDatabase; 
//            db->OpenEx("DSN=test1", CDatabase::noOdbcDialog | CDatabase::useCursorLib); printf("opendb curlib\n");
            db->OpenEx("DSN=test1", CDatabase::noOdbcDialog); printf("opendb\n");

        }
        myRS  *  t1 = new myRS(db);

        t1->Open(CRecordset::forwardOnly); printf("t.open forward\n");
//        t1->Open(CRecordset::snapshot); printf("t.open snapshot\n");
//        t1->Open(CRecordset::dynaset); printf("t.open dynaset\n");
//        t1->Open(); printf("t.open\n");


        
        switch(caseid)
        {
        case 1:
            if(t1->IsEOF()) { printf("ERROR Noting to udpate\n"); return 1; } t1->Edit(); printf("Edit\n");
            break;
        case 2:
            t1->AddNew(); printf("AddNew\n");
            break;
        default:
            printf("ERROR Unknown case %d\n", caseid); return 1;
        }

        srand(time(NULL));
//        t1->m_ID=long(rand());
        t1->m_c2_s.Format("%d", long(rand()));

        printf("before-update\n");
        t1->Update();

        printf("before-close\n");
        t1->Close();
        delete db;
        printf("OK\n");
    }catch(CException  * ee){
        char      buff[256];
        ee->GetErrorMessage(buff, sizeof buff);
        printf("ERROR %s - %s\n", locus, buff);
        ee->Delete();
        return 1;
    }

    return 0;
}
